@@ -1,6 +1,6 @@
 # uWSGI build system
 
-uwsgi_version = '1.9.18'
+uwsgi_version = '1.9.18.1'
 
 import os
 import re
@@ -627,6 +627,7 @@ class uConf(object):
         if uwsgi_os in ('Linux', 'GNU', 'GNU/kFreeBSD'):
             self.libs.append('-ldl')
         if uwsgi_os == 'GNU/kFreeBSD':
+            self.cflags.append('-D__GNU_kFreeBSD__')
             self.libs.append('-lbsd')
 
         # check for inherit option
